from enum import Enum

YEARLY = 'yearly'
QUARTERLY = 'quarterly'


USER_AGENT_HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 '
                  '(KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'
}


class TimePeriods(Enum):
    Day = '1d'
    Week = '5d'
    Month = '1mo'
    Quarter = '3mo'
    HalfYear = '6mo'
    Year = '1y'
    TwoYears = '2y'
    FiveYears = '5y'
    TenYears = '10y'
    YTD = 'ytd'
    Max = 'max'


class TimeIntervals(Enum):
    Minute = '1m'
    TwoMinutes = '2m'
    FiveMinutes = '5m'
    FifteenMinutes = '15m'
    HalfHour = '30m'
    Hourly = '60m'
    HourAndAHalf = '90m'
    Daily = '1d'
    FiveDays = '5d'
    Weekly = '1wk'
    Monthly = '1mo'
    Quarterly = '3mo'


class HistoryColumns:
    Date = 'Date'
    Open = 'Open'
    High = 'High'
    Low = 'Low'
    Close = 'Close'
    Volume = 'Volume'
    Dividends = 'Dividends'
    Splits = 'Stock Splits'


class BalanceSheetColumns:
    TotalLiabilities = 'Total Liab'
    OtherLiabilities = 'Other Liab'
    TotalCurrentLiabilities = 'Total Current Liabilities'
    OtherCurrentLiabilities = 'Other Current Liab'

    TotalAssets = 'Total Assets'
    OtherAssets = 'Other Assets'
    TotalCurrentAssets = 'Total Current Assets'
    OtherCurrentAssets = 'Other Current Assets'
    NetTangibleAssets = 'Net Tangible Assets'

    TotalStockholderEquity = 'Total Stockholder Equity'
    RetainedEarnings = 'Retained Earnings'
    CommonStock = 'Common Stock'
    TreasureStock = 'Treasury Stock'
    Cash = 'Cash'
    LongTermDebt = 'Long Term Debt'
    ShortLongTermDebt = 'Short Long Term Debt'
    OtherStockholderEquity = 'Other Stockholder Equity'
    PropertyPlantEquipment = 'Property Plant Equipment'
    LongTermInvestments = 'Long Term Investments'
    ShortTermInvestments = 'Short Term Investments'
    NetReceivables = 'Net Receivables'
    Inventory = 'Inventory'
    AccountsPayable = 'Accounts Payable'


class FinancialColumns:
    RnD = 'Research Development'
    EffectOfAccountingCharges = 'Effect Of Accounting Charges'
    IncomeBeforeTax = 'Income Before Tax'
    MinorityInterest = 'Minority Interest'
    NetIncome = 'Net Income'
    SellingGeneralAdministrative = 'Selling General Administrative'
    GrossProfit = 'Gross Profit'
    Ebit = 'Ebit'
    OperatingIncome = 'Operating Income'
    OtherOperatingExpenses = 'Other Operating Expenses'
    InterestExpense = 'Interest Expense'
    ExtraordinaryItems = 'Extraordinary Items'
    NonRecurring = 'Non Recurring'
    OtherItems = 'Other Items'
    IncomeTaxExpense = 'Income Tax Expense'
    TotalRevenue = 'Total Revenue'
    TotalOperatingExpenses = 'Total Operating Expenses'
    CostOfRevenue = 'Cost Of Revenue'
    TotalOtherIncomeExpenseNet = 'Total Other Income Expense Net'
    DiscontinuedOperations = 'Discontinued Operations'
    NetIncomeFromContinuingOps = 'Net Income From Continuing Ops'
    NetIncomeApplicableToCommonShares = 'Net Income Applicable To Common Shares'


class CashflowColumns:
    Investments = 'Investments'
    ChangeToLiabilities = 'Change To Liabilities'
    TotalCashflowFromInvestingActivities = 'Total Cashflows From Investing Activities'
    NetBorrowings = 'Net Borrowings'
    TotalCashFromFinancingActivities = 'Total Cash From Financing Activities'
    ChangeToOperatingActivities = 'Change To Operating Activities'
    IssuanceOfStock = 'Issuance Of Stock'
    NetIncome = 'Net Income'
    ChangeInCash = 'Change In Cash'
    RepurchaseOfStock = 'Repurchase Of Stock'
    TotalCashFromOperatingActivities = 'Total Cash From Operating Activities'
    Depreciation = 'Depreciation'
    DividendsPaid = 'Dividends Paid'
    ChangeToInventory = 'Change To Inventory'
    ChangeToAccountReceivables = 'Change To Account Receivables'
    OtherCashflowFromFinancingActivities = 'Other Cashflows From Financing Activities'
    ChangeToNetIncome = 'Change To Netincome'
    CapitalExpenditures = 'Capital Expenditures'


class TickerInfoKeys:
    zip = 'zip'
    sector = 'sector'
    fullTimeEmployees = 'fullTimeEmployees'
    longBusinessSummary = 'longBusinessSummary'
    city = 'city'
    phone = 'phone'
    state = 'state'
    country = 'country'
    companyOfficers = 'companyOfficers'
    website = 'website'
    maxAge = 'maxAge'
    address1 = 'address1'
    industry = 'industry'
    previousClose = 'previousClose'
    regularMarketOpen = 'regularMarketOpen'
    twoHundredDayAverage = 'twoHundredDayAverage'
    trailingAnnualDividendYield = 'trailingAnnualDividendYield'
    payoutRatio = 'payoutRatio'
    volume24Hr = 'volume24Hr'
    regularMarketDayHigh = 'regularMarketDayHigh'
    navPrice = 'navPrice'
    averageDailyVolume10Day = 'averageDailyVolume10Day'
    totalAssets = 'totalAssets'
    regularMarketPreviousClose = 'regularMarketPreviousClose'
    fiftyDayAverage = 'fiftyDayAverage'
    trailingAnnualDividendRate = 'trailingAnnualDividendRate'
    open = 'open'
    toCurrency = 'toCurrency'
    averageVolume10days = 'averageVolume10days'
    expireDate = 'expireDate'
    algorithm = 'algorithm'
    dividendRate = 'dividendRate'
    exDividendDate = 'exDividendDate'
    beta = 'beta'
    circulatingSupply = 'circulatingSupply'
    startDate = 'startDate'
    regularMarketDayLow = 'regularMarketDayLow'
    priceHint = 'priceHint'
    currency = 'currency'
    trailingPE = 'trailingPE'
    regularMarketVolume = 'regularMarketVolume'
    lastMarket = 'lastMarket'
    maxSupply = 'maxSupply'
    openInterest = 'openInterest'
    marketCap = 'marketCap'
    volumeAllCurrencies = 'volumeAllCurrencies'
    strikePrice = 'strikePrice'
    averageVolume = 'averageVolume'
    priceToSalesTrailing12Months = 'priceToSalesTrailing12Months'
    dayLow = 'dayLow'
    ask = 'ask'
    ytdReturn = 'ytdReturn'
    askSize = 'askSize'
    volume = 'volume'
    fiftyTwoWeekHigh = 'fiftyTwoWeekHigh'
    forwardPE = 'forwardPE'
    fromCurrency = 'fromCurrency'
    fiveYearAvgDividendYield = 'fiveYearAvgDividendYield'
    fiftyTwoWeekLow = 'fiftyTwoWeekLow'
    bid = 'bid'
    tradeable = 'tradeable'
    dividendYield = 'dividendYield'
    bidSize = 'bidSize'
    dayHigh = 'dayHigh'
    exchange = 'exchange'
    shortName = 'shortName'
    longName = 'longName'
    exchangeTimezoneName = 'exchangeTimezoneName'
    exchangeTimezoneShortName = 'exchangeTimezoneShortName'
    isEsgPopulated = 'isEsgPopulated'
    gmtOffSetMilliseconds = 'gmtOffSetMilliseconds'
    quoteType = 'quoteType'
    symbol = 'symbol'
    messageBoardId = 'messageBoardId'
    market = 'market'
    annualHoldingsTurnover = 'annualHoldingsTurnover'
    enterpriseToRevenue = 'enterpriseToRevenue'
    beta3Year = 'beta3Year'
    enterpriseToEbitda = 'enterpriseToEbitda'
    OneYearChange = '52WeekChange'
    morningStarRiskRating = 'morningStarRiskRating'
    forwardEps = 'forwardEps'
    revenueQuarterlyGrowth = 'revenueQuarterlyGrowth'
    sharesOutstanding = 'sharesOutstanding'
    fundInceptionDate = 'fundInceptionDate'
    annualReportExpenseRatio = 'annualReportExpenseRatio'
    bookValue = 'bookValue'
    sharesShort = 'sharesShort'
    sharesPercentSharesOut = 'sharesPercentSharesOut'
    fundFamily = 'fundFamily'
    lastFiscalYearEnd = 'lastFiscalYearEnd'
    heldPercentInstitutions = 'heldPercentInstitutions'
    netIncomeToCommon = 'netIncomeToCommon'
    trailingEps = 'trailingEps'
    lastDividendValue = 'lastDividendValue'
    SandP52WeekChange = 'SandP52WeekChange'
    priceToBook = 'priceToBook'
    heldPercentInsiders = 'heldPercentInsiders'
    nextFiscalYearEnd = 'nextFiscalYearEnd'
    mostRecentQuarter = 'mostRecentQuarter'
    shortRatio = 'shortRatio'
    sharesShortPreviousMonthDate = 'sharesShortPreviousMonthDate'
    floatShares = 'floatShares'
    enterpriseValue = 'enterpriseValue'
    threeYearAverageReturn = 'threeYearAverageReturn'
    lastSplitDate = 'lastSplitDate'
    lastSplitFactor = 'lastSplitFactor'
    legalType = 'legalType'
    morningStarOverallRating = 'morningStarOverallRating'
    earningsQuarterlyGrowth = 'earningsQuarterlyGrowth'
    dateShortInterest = 'dateShortInterest'
    pegRatio = 'pegRatio'
    lastCapGain = 'lastCapGain'
    shortPercentOfFloat = 'shortPercentOfFloat'
    sharesShortPriorMonth = 'sharesShortPriorMonth'
    category = 'category'
    fiveYearAverageReturn = 'fiveYearAverageReturn'
    regularMarketPrice = 'regularMarketPrice'
    logo_url = 'logo_url'
    ebitdaMargins = 'ebitdaMargins'
    profitMargins = 'profitMargins'
    grossMargins = 'grossMargins'
    operatingCashflow ='operatingCashflow'
    revenueGrowth = 'revenueGrowth'
    operatingMargins = 'operatingMargins'
    ebitda = 'ebitda'
    targetLowPrice = 'targetLowPrice'
    recommendationKey = 'recommendationKey'
    grossProfits = 'grossProfits'
    freeCashflow = 'freeCashflow'
    targetMedianPrice = 'targetMedianPrice'
    currentPrice = 'currentPrice'
    earningsGrowth = 'earningsGrowth'
    currentRatio = 'currentRatio'
    returnOnAssets = 'returnOnAssets'
    numberOfAnalystOpinions = 'numberOfAnalystOpinions'
    targetMeanPrice = 'targetMeanPrice'
    debtToEquity = 'debtToEquity'
    returnOnEquity = 'returnOnEquity'
    targetHighPrice = 'targetHighPrice'
    totalCash = 'totalCash'
    totalDebt = 'totalDebt'
    totalRevenue = 'totalRevenue'
    totalCashPerShare = 'totalCashPerShare'
    financialCurrency = 'financialCurrency'
    revenuePerShare = 'revenuePerShare'
    quickRatio = 'quickRatio'
    recommendationMean = 'recommendationMean'


class SustainabilityColumns:
    palmOil = 'palmOil'
    controversialWeapons = 'controversialWeapons'
    gambling = 'gambling'
    socialScore = 'socialScore'
    nuclear = 'nuclear'
    furLeather = 'furLeather'
    alcoholic = 'alcoholic'
    gmo = 'gmo'
    catholic = 'catholic'
    socialPercentile = 'socialPercentile'
    peerCount = 'peerCount'
    governanceScore = 'governanceScore'
    environmentPercentile = 'environmentPercentile'
    animalTesting = 'animalTesting'
    tobacco = 'tobacco'
    totalEsg = 'totalEsg'
    highestControversy = 'highestControversy'
    esgPerformance = 'esgPerformance'
    coal = 'coal'
    pesticides = 'pesticides'
    adult = 'adult'
    percentile = 'percentile'
    peerGroup = 'peerGroup'
    smallArms = 'smallArms'
    environmentScore = 'environmentScore'
    governancePercentile = 'governancePercentile'
    militaryContract = 'militaryContract'


class InstitutionalHoldersColumns:
    Holder = 'Holder'
    Shares = 'Shares'
    DateReported = 'Date Reported'
    Percentage = '% Out'
    Value = 'Value'


class MajorHoldersKeys(Enum):
    PctSharesByInsiders = 0
    PctSharesByInstitutions = 1
    PctFloatByInstitutions = 2
    NumInstitutionsHolding = 3


class RecommendationColumns:
    Date = 'Date'
    Firm = 'Firm'
    ToGrade = 'To Grade'
    FromGrade = 'From Grade'
    Action = 'Action'


class RecommendationGrades:
    # Warning: recommend checking these keys in a case insensitive manner
    Buy = 'Buy'
    Outperform = 'Outperform'
    Overweight = 'Overweight'
    SectorPerform = 'Sector Perform'
    Neutral = 'Neutral'
    SectorOutperform = 'Sector Outperform'
    Hold = 'Hold'
    MarketPerform = 'Market Perform'
    StrongBuy = 'Strong Buy'
    LongTermBuy = 'Long-term Buy'
    Sell = 'Sell'
    MarketOutperform = 'Market Outperform'
    Positive = 'Positive'
    EqualWeight = 'Equal-weight'
    Perform = 'Perform'
    Negative = 'Negative'
    SectorWeight = 'Sector Weight'
    Reduce = 'Reduce'
    Underweight = 'Underweight'


class EarningsColumns:
    Quarter = 'Quarter'
    Revenue = 'Revenue'
    Earnings = 'Earnings'



from unittest.mock import patch
import pandas as pd
import pytest

from yfinance_ez.base import TickerBase


def test_TickerBase_init(ticker_symbol):
    ticker = TickerBase(ticker_symbol.lower())

    assert ticker.ticker == ticker_symbol
    assert ticker._historical_data.empty
    assert ticker._fundamentals_data is None
    assert ticker._financials_data is None


@patch('yfinance_ez.base.requests')
def test_TickerBase_get_history(requests_mock, ticker_symbol, historical_data_resp):
    requests_mock.get.return_value = historical_data_resp

    ticker = TickerBase(ticker_symbol)
    ticker.get_history()

    assert isinstance(ticker._historical_data, pd.DataFrame)


@patch('yfinance_ez.base.utils')
def test_TickerBase_fundamentals_data(utils_mock, ticker_symbol):
    utils_mock.get_json.return_value = 'Fake Fundamentals Data'

    ticker = TickerBase(ticker_symbol)
    assert ticker._fundamentals_data is None

    assert ticker.fundamentals_data == 'Fake Fundamentals Data'
    assert utils_mock.get_json.called


@patch('yfinance_ez.base.utils')
def test_TickerBase_financials_data(utils_mock, ticker_symbol):
    utils_mock.get_json.return_value = 'Fake Financials Data'

    ticker = TickerBase(ticker_symbol)
    assert ticker._financials_data is None

    assert ticker.financials_data == 'Fake Financials Data'
    assert utils_mock.get_json.called


@patch('yfinance_ez.base.requests')
def test_TickerBase_get_history_fails_yahoo_down(requests_mock, ticker_symbol, yahoo_down_resp):
    requests_mock.get.return_value = yahoo_down_resp

    ticker = TickerBase(ticker_symbol)
    with pytest.raises(RuntimeError):
        ticker.get_history()



import requests as requests
import pandas as pd
import re

from yfinance_ez.constants import USER_AGENT_HEADERS

try:
    import ujson as _json
except ImportError:
    import json as _json





def get_json(url, proxy=None):
    html = requests.get(url=url, proxies=proxy, headers=USER_AGENT_HEADERS).text

    if "QuoteSummaryStore" not in html:
        html = requests.get(url=url, proxies=proxy, headers=USER_AGENT_HEADERS).text
        if "QuoteSummaryStore" not in html:
            return {}

    json_str = html.split('root.App.main =')[1].split(
        '(this)')[0].split(';\n}')[0].strip()
    data = _json.loads(json_str)[
        'context']['dispatcher']['stores']['QuoteSummaryStore']

    # return data
    new_data = _json.dumps(data).replace('{}', 'null')
    new_data = re.sub(
        r'\{[\'|\"]raw[\'|\"]:(.*?),(.*?)\}', r'\1', new_data)

    return _json.loads(new_data)


def camel2title(o):
    return [re.sub("([a-z])([A-Z])", "\g<1> \g<2>", i).title() for i in o]


def auto_adjust(data):
    df = data.copy()
    ratio = df["Close"] / df["Adj Close"]
    df["Adj Open"] = df["Open"] / ratio
    df["Adj High"] = df["High"] / ratio
    df["Adj Low"] = df["Low"] / ratio

    df.drop(
        ["Open", "High", "Low", "Close"],
        axis=1, inplace=True)

    df.rename(columns={
        "Adj Open": "Open", "Adj High": "High",
        "Adj Low": "Low", "Adj Close": "Close"
    }, inplace=True)

    df = df[["Open", "High", "Low", "Close", "Volume"]]
    return df[["Open", "High", "Low", "Close", "Volume"]]


def back_adjust(data):
    """ back-adjusted data to mimic true historical prices """

    df = data.copy()
    ratio = df["Adj Close"] / df["Close"]
    df["Adj Open"] = df["Open"] * ratio
    df["Adj High"] = df["High"] * ratio
    df["Adj Low"] = df["Low"] * ratio

    df.drop(
        ["Open", "High", "Low", "Adj Close"],
        axis=1, inplace=True)

    df.rename(columns={
        "Adj Open": "Open", "Adj High": "High",
        "Adj Low": "Low"
    }, inplace=True)

    return df[["Open", "High", "Low", "Close", "Volume"]]


def parse_quotes(data, tz=None):
    timestamps = data["timestamp"]
    ohlc = data["indicators"]["quote"][0]
    volumes = ohlc["volume"]
    opens = ohlc["open"]
    closes = ohlc["close"]
    lows = ohlc["low"]
    highs = ohlc["high"]

    adjclose = closes
    if "adjclose" in data["indicators"]:
        adjclose = data["indicators"]["adjclose"][0]["adjclose"]

    quotes = pd.DataFrame({"Open": opens,
                           "High": highs,
                           "Low": lows,
                           "Close": closes,
                           "Adj Close": adjclose,
                           "Volume": volumes})

    quotes.index = pd.to_datetime(timestamps, unit="s")
    quotes.sort_index(inplace=True)

    if tz is not None:
        quotes.index = quotes.index.tz_localize(tz)

    return quotes


def parse_actions(data, tz=None):
    dividends = pd.DataFrame(columns=["Dividends"])
    splits = pd.DataFrame(columns=["Stock Splits"])

    if "events" in data:
        if "dividends" in data["events"]:
            dividends = pd.DataFrame(
                data=list(data["events"]["dividends"].values()))
            dividends.set_index("date", inplace=True)
            dividends.index = pd.to_datetime(dividends.index, unit="s")
            dividends.sort_index(inplace=True)
            if tz is not None:
                dividends.index = dividends.index.tz_localize(tz)

            dividends.columns = ["Dividends"]

        if "splits" in data["events"]:
            splits = pd.DataFrame(
                data=list(data["events"]["splits"].values()))
            splits.set_index("date", inplace=True)
            splits.index = pd.to_datetime(splits.index, unit="s")
            splits.sort_index(inplace=True)
            if tz is not None:
                splits.index = splits.index.tz_localize(tz)
            splits["Stock Splits"] = splits["numerator"] / \
                                     splits["denominator"]
            splits = splits["Stock Splits"]

    return dividends, splits
